# -*- coding: utf-8 -*-
"""Implementation of bandit security testing in Flake8.

Bandit is a security linter for python code and needs to be installed
for this flake8 extension to work properly.
"""
import os

import pycodestyle
from bandit.core import config as b_config
from bandit.core import manager as b_manager
from flake8_polyfill import stdin

__version__ = "v0.2.0"

stdin.monkey_patch('pycodestyle')


class BanditTester(object):
    """Flake8 class for checking code for bandit test errors.

    This class is necessary and used by flake8 to check the python
    file or files that are being tested.

    """

    name = "flake8-bandit"
    version = __version__

    def __init__(self, tree, filename):
        """Initialize all the necessary attributes for flake8."""
        self.filename = filename
        self.tree = tree
        self._load_source()

    def _check_source(self):
        b_conf = b_config.BanditConfig()
        b_mgr = b_manager.BanditManager(b_conf, 'file', False)
        # it is happening here
        b_mgr.discover_files([self.filename])
        b_mgr.run_tests()
        issues = []
        for item in b_mgr.get_issue_list():
            i = {}
            i["test_id"] = item.test_id
            i["issue_text"] = item.text
            i["line_number"] = item.lineno
            issues.append(i)
        os.remove("tempbanditpythonfile.py")
        return issues

    def run(self):
        """Use to run the check."""
        for error in self._check_source():
            message = "%s %s" % (error["test_id"], error["issue_text"])
            yield (error["line_number"], 0, message, type(self))

    def _load_source(self):
        """Load the source for the specified file."""
        if self.filename == "stdin":
            self.source = pycodestyle.stdin_get_value()
            with open("tempbanditpythonfile.py", "w+") as f:
                f.write(self.source)
            self.filename = "tempbanditpythonfile.py"
        else:
            with open(self.filename) as f:
                self.source = f.read()



# -*- coding: utf-8 -*-

import io
import os
import sys
from shutil import rmtree

from setuptools import setup, Command


def get_version(fname="flake8_bandit.py"):
    with open(fname) as f:
        for line in f:
            if line.startswith("__version__"):
                return eval(line.split("=")[~0])


# Package meta-data.
NAME = "flake8_bandit"
DESCRIPTION = "Automated security testing with bandit and flake8."
URL = "https://github.com/tylerwince/flake8-bandit"
EMAIL = "tyler@myndshft.com"
AUTHOR = "Tyler Wince"

# What packages are required for this module to be executed?
REQUIRED = ["flake8", "bandit"]

here = os.path.abspath(os.path.dirname(__file__))

with io.open(os.path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = "\n" + f.read()


class UploadCommand(Command):
    """Support setup.py upload."""

    description = "Build and publish the package."
    user_options = []

    @staticmethod
    def status(s):
        """Print things in bold."""
        print("\033[1m{0}\033[0m".format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status("Removing previous builds…")
            rmtree(os.path.join(here, "dist"))
        except OSError:
            pass

        self.status("Building Source and Wheel (universal) distribution…")
        os.system("{0} setup.py sdist bdist_wheel --universal".format(
            sys.executable))

        self.status("Uploading the package to PyPi via Twine…")
        os.system("twine upload dist/*")

        sys.exit()


setup(
    name=NAME,
    version=get_version(),
    description=DESCRIPTION,
    long_description=long_description,
    author=AUTHOR,
    author_email=EMAIL,
    url=URL,
    py_modules=["flake8_bandit"],
    python_requires='>=3',
    install_requires=REQUIRED,
    include_package_data=True,
    license="MIT",
    entry_points={
        "flake8.extension": [
            "B=flake8_bandit:BanditTester",
        ],
    },
    classifiers=[
        "Framework :: Flake8",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Topic :: Security",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Quality Assurance",
    ],
    cmdclass={
        "upload": UploadCommand,
    }, )
